-- Project: ACME test chip - psm control section
-- Create: 8/24/01 
-- File: psm_control.vhd
-- By: Morris Adams
--
-- Description:  This design takes Tom Friddell's schematics of the PSM 
-- ASIC and converts them to VHDL.  This includes the CNTRL block and the Faltlatch block
-- but does not include current mode buffers, level shifters or output pads.
-- The design will be synthesized to AMI C5N 0.5 micron cells in Leonardo and layed 
-- out using L-Edit.
-- This layout will be combined with other test circuits and fabricated
-- using MOSIS.  The ACME program will document the results of the test chip. 
--
library IEEE;
use IEEE.std_logic_1164.all;
--use IEEE.std_logic_arith.all;
--use IEEE.std_logic_unsigned.all;
entity psm_control is
  port (
-- inputs to digital section
    temperin : in std_logic; -- 
    dtsd : in std_logic; -- 0 = shutdown
    sysreset : in std_logic; -- 1 = reset
    vpwm : in std_logic; -- 0 = shutdown        
    ph1bypas : in std_logic; -- 
    ph2bypas : in std_logic; -- 0 = ignore bit
    vrefhi : in std_logic; -- 
    vreflo : in std_logic; -- 0 = failure
    disa : in std_logic; -- 
	disb : in std_logic; -- 
	onoffdis : in std_logic; -- 
    vpfi : in std_logic; -- 	
	ptinput : in std_logic; -- 
	pficml4 : in std_logic; -- 
	pficml3 : in std_logic; -- 
	ntimeout : in std_logic; -- 0 = no pwmclk
	pwmbypas : in std_logic; -- 1 = selects pwm clock
	mrsel : in std_logic; -- 1 = selects shutdown
	v01rssd : in std_logic; -- 
	v01sdin : in std_logic; -- 
	v01ovin : in std_logic; -- 
	v01uvin : in std_logic; -- 
	v02sdin : in std_logic; -- 
	v02ovin : in std_logic; -- 
	v02uvin : in std_logic; -- 
	v03sdin : in std_logic; -- 
	v03ovin : in std_logic; -- 
	v03uvin : in std_logic; -- 
	ps1bypas : in std_logic; -- 0 = disable supply monitor
	ps2bypas : in std_logic; -- 
	ps3bypas : in std_logic; -- 
	vinovin : in std_logic; -- 
	vinuvin : in std_logic; -- 
	catis_sd : in std_logic; -- 
	iout1uc : in std_logic; -- 
	iout1oc : in std_logic; -- 
	iout2oc : in std_logic; -- 
	iout3oc : in std_logic; -- 
	slftst : in std_logic; -- 
	fwnoen : in std_logic; --
	fwoen : in std_logic; --
	fwlatch_t : in std_logic; -- latch control signal for faltlatch block. "0" = transparent
-- outputs of digital section	
    otout : out std_logic; -- also called ot13ocml
	ovrtmpout_c2 : out std_logic; -- output from faltlatch
    otsd_tp : out std_logic; -- 
    vpwm_tp : out std_logic; -- 
    nonout : out std_logic; -- 
	ponout : out std_logic; --
	pwmon : out std_logic; -- 1 = pwm on
	pupass_c2 : out std_logic; -- output from faltlatch. 0 = failure
	cmdnout : out std_logic; --
	pficml2 : out std_logic; --
	pficml1 : out std_logic; --
	vosd_c2 : out std_logic; -- output from faltlatch
	mrfire : out std_logic; --
	vrsd_tp : out std_logic; --
	v01ov_c2 : out std_logic; --	output from faltlatch
	v01uv_c2 : out std_logic; --	output from faltlatch
	v02ov_c2 : out std_logic; --	output from faltlatch
	v02uv_c2 : out std_logic; --	output from faltlatch
	v03ov_c2 : out std_logic; --	output from faltlatch
	v03uv_c2 : out std_logic; --	output from faltlatch
	vinov_c2 : out std_logic; --	output from faltlatch
	vinuv_c2 : out std_logic; --	output from faltlatch
	catis : out std_logic; --
	io1uc_c2 : out std_logic; --	output from faltlatch
	io1oc_c2 : out std_logic; --	output from faltlatch
	io2oc_c2 : out std_logic; --	output from faltlatch
	io3oc_c2 : out std_logic; --	output from faltlatch
	tsout : out std_logic --
	   );
end psm_control;
--
architecture RTL of psm_control is
--
-- internal signal definitions
--
    signal l_otout, l_otsd_tp, l_vpwm_tp, l_pfil1, l_vpfi_n : std_logic; 
    signal l_ptinput_n, l_slftst16, l_slftst17 : std_logic;  
	signal l_pupass_n, l_slftst12, l_slftst13, l_slftst14, l_slftst15 : std_logic;
	signal l_pupass, l_slftst6, l_slftst7, l_slftst8, l_slftst9, l_slftst10 : std_logic;
	signal l_pupass_a, l_slftst1, l_slftst2, l_slftst3, l_slftst4, l_slftst5  : std_logic;
	signal l_pupass_b, l_v03ovin_n, l_v03sdin_n, l_v03uvin_n : std_logic;
	signal l_ph1bypas_n, l_iout1oc_n, l_iout1uc_n, l_iout2oc_n, l_iout3oc_n : std_logic;
	signal l_ph2bypas_n, l_v02uvin_n, l_v01ovin_n, l_v01sdin_n, l_v01uvin_n : std_logic;
	signal l_vref_nor, l_temperin_n, l_v02ovin_n, l_v02sdin_n, l_vref : std_logic;
	signal l_vrefhi_n, l_io2oc, l_io3oc : std_logic;
	signal l_vreflo_n, l_v03uv, l_vinov, l_catis_u, l_io1uc, l_io1oc : std_logic;
	signal l_vref_q, l_ps3by, l_v03ov, l_vinuv : std_logic;
    signal l_onoff_qn, l_ps2by, l_v02uv, l_fwoen_n : std_logic; 
    signal l_onoff5, l_v01ov, l_v01uv, l_v02ov : std_logic;   
	signal l_onoff4, l_overvoltage, l_ps1by : std_logic;  
	signal l_onoff3, l_vosd, l_mrsel_n, l_vmon8, l_vmon9, l_vmon10 : std_logic;  
	signal l_onoff2, l_vmon6, l_vmon4, l_vmon5, l_vmon7 : std_logic;  
	signal l_onoff1, l_pwmbypas, l_vmon1, l_vmon3, l_shutdown, l_shutdown_n : std_logic;  
	signal l_onoffdis_n, l_vrsd_tpu, l_ntimeout_n, l_pwmbypas_n, l_vmon2  : std_logic; 
	signal l_disb_n, l_ph1pass, l_pwmon, l_vosdn : std_logic;
--
begin -- architecture body
--
---------------TMON------------------------
-- 
l_otout <= temperin or sysreset; -- create "or" gate		   
otout_xlatch : process(l_otout,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then ovrtmpout_c2 <= l_otout;
	end if;
end process otout_xlatch;
otout <= l_otout; -- drive out local signal
tmon_dff : process (dtsd, sysreset) -- create falling edge dff with clear
begin
	if sysreset = '1' then
		l_otsd_tp <= '1';  
    elsif falling_edge(dtsd) then
        l_otsd_tp <= '0' ; 
    end if;
end process tmon_dff;
otsd_tp <= l_otsd_tp; -- drive out local signal
--
---------------SLFTST PAGE 1 -------------
-- 
l_vrefhi_n <= not vrefhi;
l_vreflo_n <= not vreflo;
l_vref <= l_vrefhi_n or l_vreflo_n;
slftstpage1_dff : process (l_vref, sysreset) -- create rising edge dff with clear
begin
	if sysreset = '1' then
		l_vref_q <= '0';  
    elsif rising_edge(l_vref) then
        l_vref_q <= '1' ; 
    end if;
end process slftstpage1_dff;
l_vref_nor <= l_vref_q nor l_vref;
l_ph1bypas_n <= not ph1bypas;
l_ph2bypas_n <= not ph2bypas;
l_pupass_a <= not (l_ph1bypas_n or sysreset or l_ph1pass);
l_pupass_b <= not (l_ph2bypas_n or sysreset or l_vref_nor);
l_pupass <= l_pupass_a nor l_pupass_b;
l_pupass_n <= not l_pupass;
pupass_xlatch : process(l_pupass,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then pupass_c2 <= l_pupass;
	end if;
end process pupass_xlatch;
--
-------ONOFF--------------
--
l_disb_n <= not disb;
l_onoffdis_n <= not onoffdis;
l_onoff1 <= onoffdis nor l_onoff2;
l_onoff3 <= l_onoff1 or sysreset;
onoff_dff1 : process (disa, l_onoff3) -- create rising edge dff with clear
begin
	if l_onoff3 = '1' then
		l_onoff4 <= '0';
		l_onoff2 <= '1';
    elsif rising_edge(disa) then
        l_onoff4 <= l_disb_n;
		l_onoff2 <= not l_disb_n;
    end if;
end process onoff_dff1;
cmdnout <= l_onoff4 nor l_onoffdis_n;
l_onoff5 <= l_onoffdis_n or l_onoff4;
onoff_dff2 : process (sysreset, l_onoff5) -- create falling edge dff with clear
begin
	if l_onoff5 = '1' then
		l_onoff_qn <= '1';  
    elsif falling_edge(sysreset) then
        l_onoff_qn <= '0' ; 
    end if;
end process onoff_dff2;
--
-------PFIL-----------------------------
--
l_vpfi_n <= not vpfi;
l_ptinput_n <= not ptinput;
l_pfil1 <= l_vpfi_n or l_ptinput_n;
pficml2 <= pficml3 or l_pfil1;
pfil_dff : process (l_pfil1, pficml4) -- create rising edge dff with clear
begin
	if pficml4 = '1' then
		pficml1 <= '0';
	elsif rising_edge(l_pfil1) then
        pficml1 <= '1';
	end if;
end process pfil_dff;  
--
--------CNTRL_TOP-----------------------------------
--
l_vpwm_tp <= not vpwm;
vpwm_tp <= l_vpwm_tp; -- drive out local signal
l_pwmon <= not (l_otsd_tp or sysreset or l_vpwm_tp or l_pupass_n or l_onoff_qn or 
           l_catis_u or l_vosdn or l_vrsd_tpu);
pwmon <= l_pwmon;
ponout <= l_pwmon;
nonout <= not l_pwmon;
--
---------VMON----------------------------------
--
l_ntimeout_n <= not ntimeout;
l_pwmbypas_n <= not pwmbypas;
l_pwmbypas <= pwmbypas;
l_vmon2 <= not (l_ntimeout_n or l_pwmbypas_n or l_vmon1);
l_vmon3 <= l_pwmbypas nor l_vmon1;
l_shutdown <= l_vmon2 or l_vmon3;
l_shutdown_n <= not l_shutdown;
l_vmon6 <= l_shutdown or l_vmon4 or l_vmon5;
vmon_dff1 : process (l_vmon6, sysreset) -- create rising edge dff with clear
begin
	if sysreset = '1' then
		l_vmon7 <= '0';
    elsif rising_edge(l_vmon6) then
        l_vmon7 <= '1';
    end if;
end process vmon_dff1;
l_vosd <= l_vmon7 nor l_vmon6;
l_vosdn <= not l_vosd;
vosd_xlatch : process(l_vosd,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then vosd_c2 <= l_vosd;
	end if;
end process vosd_xlatch;
l_mrsel_n <= not mrsel;
l_vmon8 <= l_shutdown_n nor l_mrsel_n;
l_vmon9 <= l_overvoltage nor mrsel;
mrfire <= l_vmon8 or l_vmon9 or l_vmon10;
l_vmon1 <= sysreset or v01sdin or l_ps1by;
l_vrsd_tpu <= not v01rssd;
vrsd_tp <= l_vrsd_tpu;
l_overvoltage <= v01ovin or l_ps1by or sysreset;
l_v01ov <= l_overvoltage;
v01ov_xlatch : process(l_v01ov,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then v01ov_c2 <= l_v01ov;
	end if;
end process v01ov_xlatch;
l_vmon10 <= not (v01uvin or l_ps1by or sysreset);
l_v01uv <= not l_vmon10;
v01uv_xlatch : process(l_v01uv,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then v01uv_c2 <= l_v01uv;
	end if;
end process v01uv_xlatch;
l_vmon4 <= not(v02sdin or l_ps2by or sysreset);
l_v02ov <= v02ovin or l_ps2by or sysreset;
v02ov_xlatch : process(l_v02ov,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then v02ov_c2 <= l_v02ov;
	end if;
end process v02ov_xlatch;
l_v02uv <= v02uvin or l_ps2by or sysreset;
v02uv_xlatch : process(l_v02uv,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then v02uv_c2 <= l_v02uv;
	end if;
end process v02uv_xlatch;
l_vmon5 <= not (v03sdin or l_ps3by or sysreset);
l_v03ov <= v03ovin or l_ps3by or sysreset;
v03ov_xlatch : process(l_v03ov,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then v03ov_c2 <= l_v03ov;
	end if;
end process v03ov_xlatch;
l_v03uv <= v03uvin or l_ps3by or sysreset;
v03uv_xlatch : process(l_v03uv,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then v03uv_c2 <= l_v03uv;
	end if;
end process v03uv_xlatch; 
l_ps1by <= not ps1bypas;
l_ps2by <= not ps2bypas;
l_ps3by <= not ps3bypas;
l_vinov <= sysreset or vinovin;
vinov_xlatch : process(l_vinov,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then vinov_c2 <= l_vinov;
	end if;
end process vinov_xlatch;
l_vinuv <= sysreset or vinuvin;
vinuv_xlatch : process(l_vinuv,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then vinuv_c2 <= l_vinuv;
	end if;
end process vinuv_xlatch;
--
---------CATIS----------------------------------------
--
catis_dff : process (catis_sd, sysreset) -- create falling edge dff with clear
begin
	if sysreset = '1' then
		l_catis_u <= '1';  
    elsif falling_edge(catis_sd) then
        l_catis_u <= '0' ; 
    end if;
end process catis_dff;
catis <= l_catis_u;
--
---------IMON-----------------------------------
--
l_io1uc <= sysreset or iout1uc;
io1uc_xlatch : process(l_io1uc,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then io1uc_c2 <= l_io1uc;
	end if;
end process io1uc_xlatch;
l_io1oc <= sysreset or iout1oc;
io1oc_xlatch : process(l_io1oc,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then io1oc_c2 <= l_io1oc;
	end if;
end process io1oc_xlatch;
l_io2oc <= sysreset or iout2oc;
io2oc_xlatch : process(l_io2oc,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then io2oc_c2 <= l_io2oc;
	end if;
end process io2oc_xlatch;
l_io3oc <= sysreset or iout3oc;
io3oc_xlatch : process(l_io3oc,fwlatch_t) -- create transparent faltlatch
begin
	if fwlatch_t = '0' then io3oc_c2 <= l_io3oc;
	end if;
end process io3oc_xlatch;
--
-------------SLFTST PAGE 3------------------------------------------------
--
l_v01ovin_n <= not v01ovin;
l_v01sdin_n <= not v01sdin;
l_v01uvin_n <= not v01uvin;
l_iout1oc_n <= not iout1oc;
l_iout1uc_n <= not iout1uc;
l_vrefhi_n <= not vrefhi;
l_temperin_n <= not temperin;
l_vreflo_n <= not vreflo;
l_v02ovin_n <= not v02ovin;
l_v02sdin_n <= not v02sdin;
l_v02uvin_n <= not v02uvin;
l_iout2oc_n <= not iout2oc;
l_iout3oc_n <= not iout3oc;
l_v03ovin_n <= not v03ovin;
l_v03sdin_n <= not v03sdin;
l_v03uvin_n <= not v03uvin;
l_slftst1 <= not (l_vrsd_tpu or v01ovin or v01sdin or l_v01uvin_n or iout1oc or l_iout1uc_n);
l_slftst4 <= not (v01rssd or l_v01ovin_n or l_v01sdin_n or v01uvin or l_iout1oc_n or iout1uc);
l_slftst7 <= not (v02ovin or v02sdin or l_v02uvin_n or iout2oc);
l_slftst9 <= not (l_v02ovin_n or l_v02sdin_n or v02uvin or l_iout2oc_n);
l_slftst12 <= not (iout3oc or v03ovin or v03sdin or l_v03uvin_n);
l_slftst14 <= not (l_v03ovin_n or l_iout3oc_n or l_v03sdin_n or v03uvin);
l_slftst2 <= l_slftst1 nor l_ps1by;
l_slftst5 <= l_slftst4 nor l_ps1by;
l_slftst8 <= l_slftst7 nor l_ps2by;
l_slftst10 <= l_slftst9 nor l_ps2by;
l_slftst13 <= l_slftst12 nor l_ps3by;
l_slftst15 <= l_slftst14 nor l_ps3by;
l_slftst3 <= l_slftst2 or vrefhi or temperin or l_vreflo_n or l_slftst8 or l_slftst13;
l_slftst6 <= l_slftst5 or l_vrefhi_n or l_temperin_n or vreflo or l_slftst10 or l_slftst15;
slftstpage3_dff1 : process (l_slftst3, slftst) -- create rising edge dff with clear
begin
	if slftst = '1' then
		l_slftst16 <= '1';
    elsif rising_edge(l_slftst3) then
        l_slftst16 <= '0';
    end if;
end process slftstpage3_dff1;
slftstpage3_dff2 : process (l_slftst6, slftst) -- create falling edge dff with clear
begin
	if slftst = '1' then
		l_slftst17 <= '1';
    elsif falling_edge(l_slftst6) then
        l_slftst17 <= '0';
    end if;
end process slftstpage3_dff2;
l_ph1pass <= not (l_slftst16 or l_slftst17); -- or l_slftst18);
--
-------------TRISTATE----------------------------------------------------
-- 
l_fwoen_n <= not fwoen;
tsout <= fwnoen nor l_fwoen_n;
--
end RTL;











































