library IEEE;
use IEEE.std_logic_1164.all;
entity psm_core_to_pads is
  port (
    LR : in std_logic; -- connects to load_reg using IDTX3 input pad
-- load input register on up edge.
-- select mode for output shift reg.  '1' = load mode
--                                                 '0' = shift mode
    SC : in std_logic; -- connects to shift_clock using IDTX3 input pad
-- clock on up edge.  Used for input and output shift registers.  
    SI : in std_logic; -- connects to serial_input using IDTX3 input pad
-- input data bit sent into shift register
    SO : out std_logic -- serial output driven with ODTXXX04 output pad
-- output data bit from output shift register
        );
end psm_core_to_pads;
--
architecture top of psm_core_to_pads is
--
-- component declarations
--
component psm_core
  port (
    load_reg : in std_logic;
    shift_clock : in std_logic;
    serial_input : in std_logic;
    serial_output : out std_logic);
  end component;
component IDTX3
  port (
    padm : in std_logic;
    qc : out std_logic);
  end component;
component ODTXXX04
  port (
    a : in std_logic;
    padm : out std_logic);
  end component;
--
-- local signal definitions
--
  signal l_load_reg : std_logic;
  signal l_shift_clock : std_logic;
  signal l_serial_input : std_logic;
  signal l_serial_output : std_logic;
--
begin
--
-- instantiate the psm_core
--
psm_digital : psm_core
  port map (
    load_reg => l_load_reg,
    shift_clock => l_shift_clock,
    serial_input => l_serial_input,
    serial_output => l_serial_output);
--
-- instantiate the load_reg input pad
--
load_reg_pad : IDTX3
  port map (
    padm => LR,
    qc => l_load_reg);
--
-- instantiate the shift_clock input pad
--
shift_clock_pad : IDTX3
  port map (
    padm => SC,
    qc => l_shift_clock);
--
-- instantiate the serial_input input pad
--
serial_input_pad : IDTX3
  port map (
    padm => SI,
    qc => l_serial_input);
--
-- instantiate the serial_output output pad
--
serial_output_pad : ODTXXX04
  port map (
    a => l_serial_output,
    padm => SO);
--
end top;