-- Project: ACME test chip - psm control serial testbench
-- Create: 9/21/01 
-- File: psm_serial_testbench.vhd
-- By: Morris Adams
--
-- Description:  This test bench feeds the boundry scan I/O of psm_shift_regs.vhd
-- The ACME asic is pin limited and this approach allows full testing using 4 pins.
-- 42 signals are shifted serially out of this testbench.  Those 42 signals are then 
-- loaded into a 42 bit wide register that drives the inputs of the psm_control.vhd 
-- circuit.  The 28 output bits of the psm_control.vhd circuit are serially shifted 
-- out and verified by this test bench.  Each line of the vector file test_vectors.txt 
-- is read and used to test and verify the UUT. 
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use std.textio.all;

entity psm_serial_testbench is
end psm_serial_testbench;

architecture TB_ARCHITECTURE of psm_serial_testbench is
-- Component declaration of psm_shift_regs which contains psm_control
component psm_shift_regs
	port(
    serial_input : in std_logic; -- test vectors from test bench or IMS tester
    shift_clock : in std_logic; -- Input and Output shift registers clocked on up edge 
    load_reg : in std_logic; -- up edge loads input register. 1 = load output shift reg
    serial_output : out std_logic); -- test results sent to test bench or IMS tester
end component;
-- constants
constant cycle:  time := 100 ns;  -- New vector every cycle, sample in the middle.
-- local signals (inputs to psm_control as read from the vector file)
	signal tb_serial_output : std_logic; -- this drives the serial input of the shift regs
	signal shift_clk : std_logic; -- used to shift all serial data (up edge)
    signal load_reg : std_logic; -- up edge loads input reg, 1 loads output shift reg, 0 shifts
    signal tb_serial_input : std_logic; -- this comes from the serial output shift regs
	signal temperin : std_logic;  -- A0	Locator on the test vector file
	signal dtsd : std_logic;	  -- A1
	signal sysreset : std_logic;  -- A2
	signal vpwm : std_logic;	  -- A3
	signal ph1bypas : std_logic;  -- A4
	signal ph2bypas : std_logic;  -- B0
	signal vrefhi : std_logic;	  -- B1
	signal vreflo : std_logic;	  -- B2
	signal disa : std_logic;	  -- B3
	signal disb : std_logic;	  -- B4
	signal onoffdis : std_logic;  -- C0
	signal vpfi : std_logic;	  -- C1
	signal ptinput : std_logic;	  -- C2
	signal pficml4 : std_logic;	  -- C3
	signal pficml3 : std_logic;	  -- C4
	signal ntimeout : std_logic;  -- D0
	signal pwmbypas : std_logic;  -- D1
	signal mrsel : std_logic;	  -- D2
	signal v01rssd : std_logic;	  -- D3
	signal v01sdin : std_logic;	  -- D4
	signal v01ovin : std_logic;	  -- E0
	signal v01uvin : std_logic;	  -- E1
	signal v02sdin : std_logic;	  -- E2
	signal v02ovin : std_logic;	  -- E3
	signal v02uvin : std_logic;	  -- E4
	signal v03sdin : std_logic;	  -- F0
	signal v03ovin : std_logic;	  -- F1
	signal v03uvin : std_logic;	  -- F2
	signal ps1bypas : std_logic;  -- F3
	signal ps2bypas : std_logic;  -- F4
	signal ps3bypas : std_logic;  -- G0
	signal vinovin : std_logic;	  -- G1
	signal vinuvin : std_logic;	  -- G2
	signal catis_sd : std_logic;  -- G3
	signal iout1uc : std_logic;	  -- G4
	signal iout1oc : std_logic;	  -- H0
	signal iout2oc : std_logic;	  -- H1
	signal iout3oc : std_logic;	  -- H2
	signal slftst : std_logic;	  -- H3
	signal fwnoen : std_logic;	  -- H4
	signal fwoen : std_logic;	  -- I0
	signal fwlatch_t : std_logic; -- I1
-- (outputs from psm_control as read from the vector file) 
	signal otout : std_logic;	     -- I2
	signal ovrtmpout_c2 : std_logic; -- I3
	signal otsd_tp : std_logic;		 -- I4
	signal vpwm_tp : std_logic;		 -- J0
	signal nonout : std_logic;		 -- J1
	signal ponout : std_logic;		 -- J2
	signal pwmon : std_logic;		 -- J3
	signal pupass_c2 : std_logic;	 -- J4
	signal cmdnout : std_logic;		 -- K0
	signal pficml2 : std_logic;		 -- K1
	signal pficml1 : std_logic;		 -- K2
	signal vosd_c2 : std_logic;		 -- K3
	signal mrfire : std_logic;		 -- K4
	signal vrsd_tp : std_logic;		 -- L0
	signal v01ov_c2 : std_logic;	 -- L1
	signal v01uv_c2 : std_logic;	 -- L2
	signal v02ov_c2 : std_logic;	 -- L3
	signal v02uv_c2 : std_logic;	 -- L4
	signal v03ov_c2 : std_logic;	 -- M0
	signal v03uv_c2 : std_logic;	 -- M1
	signal vinov_c2 : std_logic;	 -- M2
	signal vinuv_c2 : std_logic;	 -- M3
	signal catis : std_logic;		 -- M4
	signal io1uc_c2 : std_logic;	 -- N0
	signal io1oc_c2 : std_logic;	 -- N1
	signal io2oc_c2 : std_logic;	 -- N2
	signal io3oc_c2 : std_logic;	 -- N3
	signal tsout : std_logic;		 -- N4
-- open test vector file
file testvector : text open read_mode is "./test_vectors.txt";
	
-- function	to convert std_logic to string.  Used for reporting errors
	function bit_to_string(bit:std_logic) return character is
	variable result : character;
	begin
		if bit = '0' then
			result := '0';
		elsif bit = '1' then
			result := '1';
		elsif bit = 'U' then
			result := 'U';
		elsif bit = 'X' then
			result := 'X';
		elsif bit = 'Z' then
			result := 'Z';
		elsif bit = 'W' then
			result := 'W';
		elsif bit = 'L' then
			result := 'L';
		elsif bit = 'H' then
			result := 'H';
		elsif bit = '-' then
			result := '-';
		end if;
		return result;
	end;
			
begin

	-- Unit Under Test port map
	UUT : psm_shift_regs
	port map (
	serial_input => tb_serial_output,
	shift_clock => shift_clk, 
    load_reg => load_reg,
    serial_output => tb_serial_input);

-- read data from vector file and compare with actual result
	read_check : process
	variable ok : boolean := true; -- if any error is detected, ok is set to false
	variable lbuf : line; -- line buffer for reading from the vector file
	-- values read from vector file (read doesn't accept signal parameters)
	variable vec_input_bit : std_logic;	
-- expected outputs read from vector file
	variable exp_otout : std_logic;
	variable exp_ovrtmpout_c2 : std_logic;
	variable exp_otsd_tp : std_logic;
	variable exp_vpwm_tp : std_logic;
	variable exp_nonout : std_logic;
	variable exp_ponout : std_logic;
	variable exp_pwmon : std_logic;
	variable exp_pupass_c2 : std_logic;
	variable exp_cmdnout : std_logic;
	variable exp_pficml2 : std_logic;
	variable exp_pficml1 : std_logic;
	variable exp_vosd_c2 : std_logic;
	variable exp_mrfire : std_logic;
	variable exp_vrsd_tp : std_logic;
	variable exp_v01ov_c2 : std_logic;
	variable exp_v01uv_c2 : std_logic;
	variable exp_v02ov_c2 : std_logic;
	variable exp_v02uv_c2 : std_logic;
	variable exp_v03ov_c2 : std_logic;
	variable exp_v03uv_c2 : std_logic;
	variable exp_vinov_c2 : std_logic;
	variable exp_vinuv_c2 : std_logic;
	variable exp_catis : std_logic;
	variable exp_io1uc_c2 : std_logic;
	variable exp_io1oc_c2 : std_logic;
	variable exp_io2oc_c2 : std_logic;
	variable exp_io3oc_c2 : std_logic;
	variable exp_tsout : std_logic;	

	begin
		readline(testvector,lbuf); -- read and throw away 
	    readline(testvector,lbuf); -- the headers (A thru 0)	
		while not endfile(testvector) loop -- as long as there is data, keep reading
			readline(testvector,lbuf); -- transfer one line from the file to the buffer
			shift_clk <= '0'; -- initial condition of shift clock signal
			load_reg <= '0'; -- initial condition of load register signal
			for i in 1 to 42 loop
				read(lbuf,vec_input_bit);  -- read data bit from vector file
			    tb_serial_output <= vec_input_bit; -- set the signal to the variable value
			    wait for cycle/2; -- delay half of the shift clock period
			    shift_clk <= '1'; -- up edge should clock serial data into shift reg
			    wait for cycle/2; -- delay half of the shift clock period
				shift_clk <= '0'; -- down edge of clock
			end loop;  
			load_reg <= '1'; -- up edge transfers data from shift registers to input reg
			wait for cycle; -- delay full cycle for prop delay thru psm circuit
-- read expected outputs from vector file 
			read(lbuf,exp_otout);  -- read data bit from vector file
			read(lbuf,exp_ovrtmpout_c2);  -- read data bit from vector file
			read(lbuf,exp_otsd_tp);  -- read data bit from vector file
			read(lbuf,exp_vpwm_tp);  -- read data bit from vector file
			read(lbuf,exp_nonout);  -- read data bit from vector file
			read(lbuf,exp_ponout);  -- read data bit from vector file
			read(lbuf,exp_pwmon);  -- read data bit from vector file
			read(lbuf,exp_pupass_c2);  -- read data bit from vector file
			read(lbuf,exp_cmdnout);  -- read data bit from vector file
			read(lbuf,exp_pficml2);  -- read data bit from vector file
			read(lbuf,exp_pficml1);  -- read data bit from vector file
			read(lbuf,exp_vosd_c2);  -- read data bit from vector file
			read(lbuf,exp_mrfire);  -- read data bit from vector file
			read(lbuf,exp_vrsd_tp);  -- read data bit from vector file
			read(lbuf,exp_v01ov_c2);  -- read data bit from vector file
			read(lbuf,exp_v01uv_c2);  -- read data bit from vector file
			read(lbuf,exp_v02ov_c2);  -- read data bit from vector file
			read(lbuf,exp_v02uv_c2);  -- read data bit from vector file
			read(lbuf,exp_v03ov_c2);  -- read data bit from vector file
			read(lbuf,exp_v03uv_c2);  -- read data bit from vector file
			read(lbuf,exp_vinov_c2);  -- read data bit from vector file
			read(lbuf,exp_vinuv_c2);  -- read data bit from vector file
			read(lbuf,exp_catis);  -- read data bit from vector file
			read(lbuf,exp_io1uc_c2);  -- read data bit from vector file
			read(lbuf,exp_io1oc_c2);  -- read data bit from vector file
			read(lbuf,exp_io2oc_c2);  -- read data bit from vector file
			read(lbuf,exp_io3oc_c2);  -- read data bit from vector file
			read(lbuf,exp_tsout);  -- read data bit from vector file 
			-- check each output and report any errors detected
			-- if expected output is a '-' then its a don't care so don't flag an error
			shift_clk <= '1'; -- up edge should clock psm data into serial shift reg
			wait for cycle/2; --
			load_reg <= '0'; --	output register is loaded and now acts as shift register
			if ((exp_otout /= tb_serial_input) and (exp_otout /= '-')) then -- if error detected
				ok := false;			 -- clear OK flag and report problem
				report "otout should be " & bit_to_string(exp_otout) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; -- 
			wait for cycle/2; --
		    shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_ovrtmpout_c2 /= tb_serial_input) and (exp_ovrtmpout_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "ovrtmpout_c2 should be " & bit_to_string(exp_ovrtmpout_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_otsd_tp /= tb_serial_input) and (exp_otsd_tp /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "otsd_tp should be " & bit_to_string(exp_otsd_tp) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_vpwm_tp /= tb_serial_input) and (exp_vpwm_tp /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "vpwm_tp should be " & bit_to_string(exp_vpwm_tp) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_nonout /= tb_serial_input) and (exp_nonout /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "nonout should be " & bit_to_string(exp_nonout) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
	        shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_ponout /= tb_serial_input) and (exp_ponout /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "ponout should be " & bit_to_string(exp_ponout) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_pwmon /= tb_serial_input) and (exp_pwmon /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "pwmon should be " & bit_to_string(exp_pwmon) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_pupass_c2 /= tb_serial_input) and (exp_pupass_c2 /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "pupass_c2 should be " & bit_to_string(exp_pupass_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_cmdnout /= tb_serial_input) and (exp_cmdnout /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "cmdnout should be " & bit_to_string(exp_cmdnout) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_pficml2 /= tb_serial_input) and (exp_pficml2 /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "pficml2 should be " & bit_to_string(exp_pficml2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_pficml1 /= tb_serial_input) and (exp_pficml1 /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "pficml1 should be " & bit_to_string(exp_pficml1) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_vosd_c2 /= tb_serial_input) and (exp_vosd_c2 /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "vosd_c2 should be " & bit_to_string(exp_vosd_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_mrfire /= tb_serial_input) and (exp_mrfire /= '-')) then	 
				ok := false;			 -- clear OK flag and report problem
				report "mrfire should be " & bit_to_string(exp_mrfire) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_vrsd_tp /= tb_serial_input) and (exp_vrsd_tp /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "vrsd_tp should be " & bit_to_string(exp_vrsd_tp) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_v01ov_c2 /= tb_serial_input) and (exp_v01ov_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "v01ov_c2 should be " & bit_to_string(exp_v01ov_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_v01uv_c2 /= tb_serial_input) and (exp_v01uv_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "v01uv_c2 should be " & bit_to_string(exp_v01uv_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_v02ov_c2 /= tb_serial_input) and (exp_v02ov_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "v02ov_c2 should be " & bit_to_string(exp_v02ov_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_v02uv_c2 /= tb_serial_input) and (exp_v02uv_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "v02uv_c2 should be " & bit_to_string(exp_v02uv_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
	    	shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_v03ov_c2 /= tb_serial_input) and (exp_v03ov_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "v03ov_c2 should be " & bit_to_string(exp_v03ov_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_v03uv_c2 /= tb_serial_input) and (exp_v03uv_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "v03uv_c2 should be " & bit_to_string(exp_v03uv_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_vinov_c2 /= tb_serial_input) and (exp_vinov_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "vinov_c2 should be " & bit_to_string(exp_vinov_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_vinuv_c2 /= tb_serial_input) and (exp_vinuv_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "vinuv_c2 should be " & bit_to_string(exp_vinuv_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_catis /= tb_serial_input) and (exp_catis /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "catis should be " & bit_to_string(exp_catis) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_io1uc_c2 /= tb_serial_input) and (exp_io1uc_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "io1uc_c2 should be " & bit_to_string(exp_io1uc_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --  
			if ((exp_io1oc_c2 /= tb_serial_input) and (exp_io1oc_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "io1oc_c2 should be " & bit_to_string(exp_io1oc_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_io2oc_c2 /= tb_serial_input) and (exp_io2oc_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "io2oc_c2 should be " & bit_to_string(exp_io2oc_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_io3oc_c2 /= tb_serial_input) and (exp_io3oc_c2 /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "io3oc_c2 should be " & bit_to_string(exp_io3oc_c2) & 
				" was " & bit_to_string(tb_serial_input);
			end if;
			shift_clk <= '0'; --
			wait for cycle/2; --
			shift_clk <= '1'; -- 
			wait for cycle/2; --
			if ((exp_tsout /= tb_serial_input) and (exp_tsout /= '-')) then
				ok := false;			 -- clear OK flag and report problem
				report "tsout should be " & bit_to_string(exp_tsout) & 
				" was " & bit_to_string(tb_serial_input);
			end if;	
			shift_clk <= '0'; --
			wait for cycle;  -- wait a cycle before inputing new data into uut
		end loop;	 -- finished with test vector file
		if ok then 	 -- report if any errors were detected
			report "*************Testbench simulation passed with no errors*******";
		else
			report "*************Errors were detected*****************************";
		end if;
		file_close(testvector);	  -- close the vector file for future use
		wait;			 -- kill the proccess
		end process;
			
end TB_ARCHITECTURE;



